package com.example.tlsdataupdatetozzq.utils;

import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import java.security.Key;
import java.security.Security;
import java.util.Arrays;

public class Sm4Util {
    static {
        Security.addProvider(new BouncyCastleProvider());
    }
    private static final String ENCODING = "UTF-8";
    public static final String ALGORITHM_NAME = "SM4";
    // 加密算法/分组加密模式/分组填充方式
            // PKCS5Padding-以8个字节为一组进行分组加密
            // 定义分组加密模式使用：PKCS5Padding
    public static final String ALGORITHM_NAME_ECB_PADDING = "SM4/ECB/PKCS5Padding";
    /**
          * 生成ECB暗号
     */
    private static Cipher generateEcbCipher(String algorithmName, int mode, byte[] key) throws Exception {
        Cipher cipher = Cipher.getInstance(algorithmName, BouncyCastleProvider.PROVIDER_NAME);
        Key sm4Key = new SecretKeySpec(key, ALGORITHM_NAME);
        cipher.init(mode, sm4Key);
        return cipher;
    }

    /**
      * sm4加密
      * ①explain加密模式：ECB密文长度不固定，会随着被加密字符串长度的变化而变化
      * ①param Key密钥（忽略大小写）
      * @param paramStr 待加密字符串
      * @return返回16进制的加密字符串
      * @throws Exception
      */
    public static String encryptEcb(String Key, String paramStr) throws Exception {
        String cipherText = "";
        //生成16进制秘钥
        String hexKey = generateKey(Key);
        byte[] keyData = ByteUtils.fromHexString(hexKey);
        byte[] srcData = paramStr.getBytes(ENCODING);
        //加密后的数组
        byte[] cipherArray = encrypt_Ecb_Padding(keyData, srcData);
        cipherText = ByteUtils.toHexString(cipherArray);
        return cipherText;
    }

    /**
      * 加密模式之 ECB
      * @param key
      * @param data
      * @return
      * @throws Exception
      */
            public static byte[] encrypt_Ecb_Padding(byte[] key, byte[] data) throws Exception {
        Cipher cipher = generateEcbCipher(ALGORITHM_NAME_ECB_PADDING, Cipher.ENCRYPT_MODE, key);
        return cipher.doFinal(data);
    }

    /**
      * Sm4解密
      * explain解密模式：采用ECB
      * @param Key 密钥
      * @param cipherText 16进制的加密字符串（忽略大小写)
      * @return 解密后的字符串
      * @throws Exception
      */
            public static String decryptEcb(String Key, String cipherText) throws Exception {
        String hexKey = generateKey(Key);
        String decryptStr = "";
        byte[] keyData = ByteUtils.fromHexString(hexKey);
        byte[] cipherData = ByteUtils.fromHexString(cipherText);
        byte[] srcData = decrypt_Ecb_Padding(keyData, cipherData);
        decryptStr = new String(srcData,ENCODING);
        return decryptStr;
    }

    /**
      * 解密
      * @param key
      * @param cipherText
      * @return
      * @throws Exception
      */
            public static byte[] decrypt_Ecb_Padding(byte[] key, byte[] cipherText) throws Exception {
        Cipher cipher = generateEcbCipher(ALGORITHM_NAME_ECB_PADDING, Cipher.DECRYPT_MODE, key);//生成Ecb暗号，通过第二个参数判断加密还是解密
        return cipher.doFinal(cipherText);
    }

    /**
      * 校验加密前后的字符串是否为同一数据
      * @param Key 密钥（忽略大小写)
      * @param cipherText 16进制加密后的字符串
      * @param paramStr 加密前的字符串
      * @return 是否为同一数据
      * @throws Exception
      */
            public static boolean verifyEcb(String Key, String cipherText, String paramStr) throws Exception {
        boolean flag = false;
        String hexKey = generateKey(Key);
        byte[] keyData =ByteUtils.fromHexString(hexKey);
        byte[] cipherData = ByteUtils.fromHexString(cipherText);
        byte[] decryptData = decrypt_Ecb_Padding(keyData, cipherData);
        byte[] srcData = paramStr.getBytes(ENCODING);
        flag = Arrays.equals(decryptData, srcData);
        return flag;
    }

    /**
      * 产生秘钥
      * @param str
      * @return
      */
    public static String generateKey(String str){
        char[] chars = "0123456789ABCDEF".toCharArray();
        StringBuilder sb = new StringBuilder("");
        str = str + "2GOVCms";
        byte[] bs = str.getBytes();
        int bit;
        for (int i = 0; i < bs.length; i++) {
            bit = (bs[i] & 0x0f0) >> 4;
            sb.append(chars[bit]);
            bit = bs[i] & 0x0f;
            sb.append(chars[bit]);
        }
        if (sb.toString().trim().length() > 32 ) {
            return sb.toString().trim().substring(0,32);
        }
        return  sb.toString().trim();
    }

    public static void main(String[] args) throws Exception {
        String data = "内蒙古自治区人民政府办公厅关于推进马铃薯产业链发展六条政策措施的通知";
        String keys = "nmg20220712";
        //加密
        String result = encryptEcb(keys, data);
        //解密
        String dres = decryptEcb(keys,result);
        //验证
        boolean bool = verifyEcb(keys, result, data);
    }
}
